@@ -1,4 +1,4 @@
-# Copyright (C) 2004, 2005, 2006, 2007 Canonical Ltd
+# Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -188,6 +188,10 @@ def _expand_annotations(annotations, branch, current_rev=None):
         # or something.
         last_revision = current_rev.revision_id
         # XXX: Partially Cloned from branch, uses the old_get_graph, eep.
+        # XXX: The main difficulty is that we need to inject a single new node
+        #      (current_rev) into the graph before it gets numbered, etc.
+        #      Once KnownGraph gets an 'add_node()' function, we can use
+        #      VF.get_known_graph_ancestry().
         graph = repository.get_graph()
         revision_graph = dict(((key, value) for key, value in
             graph.iter_ancestry(current_rev.parent_ids) if value is not None))
@@ -313,7 +317,9 @@ def _get_matching_blocks(old, new):
     return matcher.get_matching_blocks()
 
 
-def _break_annotation_tie(annotated_lines):
+_break_annotation_tie = None
+
+def _old_break_annotation_tie(annotated_lines):
     """Chose an attribution between several possible ones.
 
     :param annotated_lines: A list of tuples ((file_id, rev_id), line) where
@@ -394,7 +400,11 @@ def _find_matching_unannotated_lines(output_lines, plain_child_lines,
                         # If the result is not stable, there is a risk a
                         # performance degradation as criss-cross merges will
                         # flip-flop the attribution.
-                        output_append(_break_annotation_tie([left, right]))
+                        if _break_annotation_tie is None:
+                            output_append(
+                                _old_break_annotation_tie([left, right]))
+                        else:
+                            output_append(_break_annotation_tie([left, right]))
         last_child_idx = child_idx + match_len
 
 
@@ -444,3 +454,10 @@ def _reannotate_annotated(right_parent_lines, new_lines, new_revision_id,
         # If left and right agree on a range, just push that into the output
         lines_extend(annotated_lines[left_idx:left_idx + match_len])
     return lines
+
+
+try:
+    from bzrlib._annotator_pyx import Annotator
+except ImportError, e:
+    osutils.failed_to_load_extension(e)
+    from bzrlib._annotator_py import Annotator
