@@ -13,47 +13,74 @@ from _lsprof import Profiler, profiler_entry
 
 __all__ = ['profile', 'Stats']
 
-_g_threadmap = {}
-
-
-def _thread_profile(f, *args, **kwds):
-    # we lose the first profile point for a new thread in order to trampoline
-    # a new Profile object into place
-    global _g_threadmap
-    thr = thread.get_ident()
-    _g_threadmap[thr] = p = Profiler()
-    # this overrides our sys.setprofile hook:
-    p.enable(subcalls=True, builtins=True)
-
-
 def profile(f, *args, **kwds):
-    """XXX docstring"""
-    global _g_threadmap
-    p = Profiler()
-    p.enable(subcalls=True)
-    threading.setprofile(_thread_profile)
-    # Note: The except clause is needed below so that profiling data still
-    # gets dumped even when exceptions are encountered. The except clause code
-    # is taken straight from run_bzr_catch_errrors() in commands.py and ought
-    # to be kept in sync with it.
+    """Run a function profile.
+
+    Exceptions are not caught: If you need stats even when exceptions are to be
+    raised, pass in a closure that will catch the exceptions and transform them
+    appropriately for your driver function.
+
+    :return: The functions return value and a stats object.
+    """
+    profiler = BzrProfiler()
+    profiler.start()
     try:
-        try:
-            ret = f(*args, **kwds)
-        except (KeyboardInterrupt, Exception), e:
-            import bzrlib.trace
-            bzrlib.trace.report_exception(sys.exc_info(), sys.stderr)
-            ret = 3
+        ret = f(*args, **kwds)
     finally:
-        p.disable()
-        for pp in _g_threadmap.values():
+        stats = profiler.stop()
+    return ret, stats
+
+
+class BzrProfiler(object):
+    """Bzr utility wrapper around Profiler.
+    
+    For most uses the module level 'profile()' function will be suitable.
+    However profiling when a simple wrapped function isn't available may
+    be easier to accomplish using this class.
+
+    To use it, create a BzrProfiler and call start() on it. Some arbitrary
+    time later call stop() to stop profiling and retrieve the statistics
+    from the code executed in the interim.
+    """
+
+    def start(self):
+        """Start profiling.
+        
+        This hooks into threading and will record all calls made until
+        stop() is called.
+        """
+        self._g_threadmap = {}
+        self.p = Profiler()
+        self.p.enable(subcalls=True)
+        threading.setprofile(self._thread_profile)
+
+    def stop(self):
+        """Stop profiling.
+
+        This unhooks from threading and cleans up the profiler, returning
+        the gathered Stats object.
+
+        :return: A bzrlib.lsprof.Stats object.
+        """
+        self.p.disable()
+        for pp in self._g_threadmap.values():
             pp.disable()
         threading.setprofile(None)
-    
-    threads = {}
-    for tid, pp in _g_threadmap.items():
-        threads[tid] = Stats(pp.getstats(), {})
-    _g_threadmap = {}
-    return ret, Stats(p.getstats(), threads)
+        p = self.p
+        self.p = None
+        threads = {}
+        for tid, pp in self._g_threadmap.items():
+            threads[tid] = Stats(pp.getstats(), {})
+        self._g_threadmap = None
+        return Stats(p.getstats(), threads)
+
+    def _thread_profile(self, f, *args, **kwds):
+        # we lose the first profile point for a new thread in order to
+        # trampoline a new Profile object into place
+        thr = thread.get_ident()
+        self._g_threadmap[thr] = p = Profiler()
+        # this overrides our sys.setprofile hook:
+        p.enable(subcalls=True, builtins=True)
 
 
 class Stats(object):
@@ -153,7 +180,7 @@ class _CallTreeFilter(object):
 
     This code is taken from http://ddaa.net/blog/python/lsprof-calltree
     with the changes made by J.P. Calderone and Itamar applied. Note that
-    isinstance(code, str) needs to be used at times to determine if the code 
+    isinstance(code, str) needs to be used at times to determine if the code
     object is actually an external code object (with a filename, etc.) or
     a Python built-in.
     """
@@ -163,7 +190,7 @@ class _CallTreeFilter(object):
         self.out_file = None
 
     def output(self, out_file):
-        self.out_file = out_file        
+        self.out_file = out_file
         out_file.write('events: Ticks\n')
         self._print_summary()
         for entry in self.data:
