@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 
 """bzr python plugin support.
@@ -41,7 +41,6 @@ lazy_import(globals(), """
 import imp
 import re
 import types
-import zipfile
 
 from bzrlib import (
     _format_version_tuple,
@@ -53,12 +52,16 @@ from bzrlib import (
 from bzrlib import plugins as _mod_plugins
 """)
 
-from bzrlib.symbol_versioning import deprecated_function, one_three
+from bzrlib.symbol_versioning import (
+    deprecated_function,
+    deprecated_in,
+    )
 
 
 DEFAULT_PLUGIN_PATH = None
 _loaded = False
 
+@deprecated_function(deprecated_in((2, 0, 0)))
 def get_default_plugin_path():
     """Get the DEFAULT_PLUGIN_PATH"""
     global DEFAULT_PLUGIN_PATH
@@ -92,13 +95,15 @@ def set_plugins_path(path=None):
     return path
 
 
-def get_standard_plugins_path():
-    """Determine a plugin path suitable for general use."""
-    path = os.environ.get('BZR_PLUGIN_PATH',
-                          get_default_plugin_path()).split(os.pathsep)
-    # Get rid of trailing slashes, since Python can't handle them when
-    # it tries to import modules.
-    path = map(_strip_trailing_sep, path)
+def _append_new_path(paths, new_path):
+    """Append a new path if it set and not already known."""
+    if new_path is not None and new_path not in paths:
+        paths.append(new_path)
+    return paths
+
+
+def get_core_plugin_path():
+    core_path = None
     bzr_exe = bool(getattr(sys, 'frozen', None))
     if bzr_exe:    # expand path for bzr.exe
         # We need to use relative path to system-wide plugin
@@ -111,25 +116,83 @@ def get_standard_plugins_path():
         # then plugins directory is
         # C:\Program Files\Bazaar\plugins
         # so relative path is ../../../plugins
-        path.append(osutils.abspath(osutils.pathjoin(
-            osutils.dirname(__file__), '../../../plugins')))
-    if not bzr_exe:     # don't look inside library.zip
+        core_path = osutils.abspath(osutils.pathjoin(
+                osutils.dirname(__file__), '../../../plugins'))
+    else:     # don't look inside library.zip
         # search the plugin path before the bzrlib installed dir
-        path.append(os.path.dirname(_mod_plugins.__file__))
-    # search the arch independent path if we can determine that and
-    # the plugin is found nowhere else
-    if sys.platform != 'win32':
-        try:
-            from distutils.sysconfig import get_python_lib
-        except ImportError:
-            # If distutuils is not available, we just won't add that path
-            pass
-        else:
-            archless_path = osutils.pathjoin(get_python_lib(), 'bzrlib',
-                    'plugins')
-            if archless_path not in path:
-                path.append(archless_path)
-    return path
+        core_path = os.path.dirname(_mod_plugins.__file__)
+    return core_path
+
+
+def get_site_plugin_path():
+    """Returns the path for the site installed plugins."""
+    if sys.platform == 'win32':
+        # We don't have (yet) a good answer for windows since that is certainly
+        # related to the way we build the installers. -- vila20090821
+        return None
+    site_path = None
+    try:
+        from distutils.sysconfig import get_python_lib
+    except ImportError:
+        # If distutuils is not available, we just don't know where they are
+        pass
+    else:
+        site_path = osutils.pathjoin(get_python_lib(), 'bzrlib', 'plugins')
+    return site_path
+
+
+def get_user_plugin_path():
+    return osutils.pathjoin(config.config_dir(), 'plugins')
+
+
+def get_standard_plugins_path():
+    """Determine a plugin path suitable for general use."""
+    # Ad-Hoc default: core is not overriden by site but user can overrides both
+    # The rationale is that:
+    # - 'site' comes last, because these plugins should always be available and
+    #   are supposed to be in sync with the bzr installed on site.
+    # - 'core' comes before 'site' so that running bzr from sources or a user
+    #   installed version overrides the site version.
+    # - 'user' comes first, because... user is always right.
+    # - the above rules clearly defines which plugin version will be loaded if
+    #   several exist. Yet, it is sometimes desirable to disable some directory
+    #   so that a set of plugins is disabled as once. This can be done via
+    #   -site, -core, -user.
+
+    env_paths = os.environ.get('BZR_PLUGIN_PATH', '+user').split(os.pathsep)
+    defaults = ['+core', '+site']
+
+    # The predefined references
+    refs = dict(core=get_core_plugin_path(),
+                site=get_site_plugin_path(),
+                user=get_user_plugin_path())
+
+    # Unset paths that should be removed
+    for k,v in refs.iteritems():
+        removed = '-%s' % k
+        # defaults can never mention removing paths as that will make it
+        # impossible for the user to revoke these removals.
+        if removed in env_paths:
+            env_paths.remove(removed)
+            refs[k] = None
+
+    # Expand references
+    paths = []
+    for p in env_paths + defaults:
+        if p.startswith('+'):
+            # Resolve reference if they are known
+            try:
+                p = refs[p[1:]]
+            except KeyError:
+                # Leave them untouched otherwise, user may have paths starting
+                # with '+'...
+                pass
+        _append_new_path(paths, p)
+
+    # Get rid of trailing slashes, since Python can't handle them when
+    # it tries to import modules.
+    paths = map(_strip_trailing_sep, paths)
+    return paths
 
 
 def load_plugins(path=None):
@@ -254,87 +317,6 @@ def load_from_dir(d):
                 trace.print_exception(sys.exc_info(), sys.stderr)
 
 
-@deprecated_function(one_three)
-def load_from_zip(zip_name):
-    """Load all the plugins in a zip."""
-    valid_suffixes = ('.py', '.pyc', '.pyo')    # only python modules/packages
-                                                # is allowed
-    try:
-        index = zip_name.rindex('.zip')
-    except ValueError:
-        return
-    archive = zip_name[:index+4]
-    prefix = zip_name[index+5:]
-
-    trace.mutter('Looking for plugins in %r', zip_name)
-
-    # use zipfile to get list of files/dirs inside zip
-    try:
-        z = zipfile.ZipFile(archive)
-        namelist = z.namelist()
-        z.close()
-    except zipfile.error:
-        # not a valid zip
-        return
-
-    if prefix:
-        prefix = prefix.replace('\\','/')
-        if prefix[-1] != '/':
-            prefix += '/'
-        ix = len(prefix)
-        namelist = [name[ix:]
-                    for name in namelist
-                    if name.startswith(prefix)]
-
-    trace.mutter('Names in archive: %r', namelist)
-
-    for name in namelist:
-        if not name or name.endswith('/'):
-            continue
-
-        # '/' is used to separate pathname components inside zip archives
-        ix = name.rfind('/')
-        if ix == -1:
-            head, tail = '', name
-        else:
-            head, tail = name.rsplit('/',1)
-        if '/' in head:
-            # we don't need looking in subdirectories
-            continue
-
-        base, suffix = osutils.splitext(tail)
-        if suffix not in valid_suffixes:
-            continue
-
-        if base == '__init__':
-            # package
-            plugin_name = head
-        elif head == '':
-            # module
-            plugin_name = base
-        else:
-            continue
-
-        if not plugin_name:
-            continue
-        if getattr(_mod_plugins, plugin_name, None):
-            trace.mutter('Plugin name %s already loaded', plugin_name)
-            continue
-
-        try:
-            exec "import bzrlib.plugins.%s" % plugin_name in {}
-            trace.mutter('Load plugin %s from zip %r', plugin_name, zip_name)
-        except KeyboardInterrupt:
-            raise
-        except Exception, e:
-            ## import pdb; pdb.set_trace()
-            trace.warning('Unable to load plugin %r from %r'
-                    % (name, zip_name))
-            trace.log_exception_quietly()
-            if 'error' in debug.debug_flags:
-                trace.print_exception(sys.exc_info(), sys.stderr)
-
-
 def plugins():
     """Return a dictionary of the plugins.
 
@@ -385,7 +367,7 @@ class ModuleHelpTopic(object):
         """
         self.module = module
 
-    def get_help_text(self, additional_see_also=None):
+    def get_help_text(self, additional_see_also=None, verbose=True):
         """Return a string with the help for this topic.
 
         :param additional_see_also: Additional help topics to be
