@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2007, 2009 Canonical Ltd
+# Copyright (C) 2006-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -27,6 +27,7 @@ from bzrlib.tests import (
     TestCaseInTempDir,
     TestCaseWithTransport,
     TestUIFactory,
+    posix_permissions_feature,
     )
 from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer
 from bzrlib.transport import get_transport
@@ -42,29 +43,35 @@ class TestWithUpgradableBranches(TestCaseWithTransport):
         self.old_format = bzrdir.BzrDirFormat.get_default_format()
         self.old_ui_factory = ui.ui_factory
         self.addCleanup(self.restoreDefaults)
-
         ui.ui_factory = TestUIFactory()
-        # setup a format 5 branch we can upgrade from.
-        self.make_branch_and_tree('format_5_branch',
-                                  format=bzrdir.BzrDirFormat5())
-
-        current_tree = self.make_branch_and_tree('current_format_branch',
-                                                 format='default')
-        self.make_branch_and_tree('metadir_weave_branch', format='metaweave')
-        current_tree.branch.create_checkout(
-            self.get_url('current_format_checkout'), lightweight=True)
 
     def restoreDefaults(self):
         ui.ui_factory = self.old_ui_factory
         bzrdir.BzrDirFormat._set_default_format(self.old_format)
 
+    def make_current_format_branch_and_checkout(self):
+        current_tree = self.make_branch_and_tree('current_format_branch',
+                                                 format='default')
+        current_tree.branch.create_checkout(
+            self.get_url('current_format_checkout'), lightweight=True)
+
+    def make_format_5_branch(self):
+        # setup a format 5 branch we can upgrade from.
+        self.make_branch_and_tree('format_5_branch',
+                                  format=bzrdir.BzrDirFormat5())
+
+    def make_metadir_weave_branch(self):
+        self.make_branch_and_tree('metadir_weave_branch', format='metaweave')
+
     def test_readonly_url_error(self):
+        self.make_format_5_branch()
         (out, err) = self.run_bzr(
             ['upgrade', self.get_readonly_url('format_5_branch')], retcode=3)
         self.assertEqual(out, "")
         self.assertEqual(err, "bzr: ERROR: Upgrade URL cannot work with readonly URLs.\n")
 
     def test_upgrade_up_to_date(self):
+        self.make_current_format_branch_and_checkout()
         # when up to date we should get a message to that effect
         (out, err) = self.run_bzr('upgrade current_format_branch', retcode=3)
         self.assertEqual("", out)
@@ -73,6 +80,7 @@ class TestWithUpgradableBranches(TestCaseWithTransport):
                              "recent format.\n", err)
 
     def test_upgrade_up_to_date_checkout_warns_branch_left_alone(self):
+        self.make_current_format_branch_and_checkout()
         # when upgrading a checkout, the branch location and a suggestion
         # to upgrade it should be emitted even if the checkout is up to
         # date
@@ -100,6 +108,7 @@ class TestWithUpgradableBranches(TestCaseWithTransport):
 
     def test_upgrade_explicit_metaformat(self):
         # users can force an upgrade to metadir format.
+        self.make_format_5_branch()
         url = get_transport(self.get_url('format_5_branch')).base
         # check --format takes effect
         bzrdir.BzrDirFormat._set_default_format(bzrdir.BzrDirFormat5())
@@ -122,6 +131,7 @@ finished
     def test_upgrade_explicit_knit(self):
         # users can force an upgrade to knit format from a metadir weave
         # branch
+        self.make_metadir_weave_branch()
         url = get_transport(self.get_url('metadir_weave_branch')).base
         # check --format takes effect
         bzrdir.BzrDirFormat._set_default_format(bzrdir.BzrDirFormat5())
@@ -145,6 +155,18 @@ finished
         self.run_bzr('init-repository --format=metaweave repo')
         self.run_bzr('upgrade --format=knit repo')
 
+    def test_upgrade_permission_check(self):
+        """'backup.bzr' should retain permissions of .bzr. Bug #262450"""
+        self.requireFeature(posix_permissions_feature)
+        import os, stat
+        old_perms = stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR
+        backup_dir = 'backup.bzr'
+        self.run_bzr('init --format=1.6')
+        os.chmod('.bzr', old_perms)
+        self.run_bzr('upgrade')
+        new_perms = os.stat(backup_dir).st_mode & 0777
+        self.assertTrue(new_perms == old_perms)
+
 
 class SFTPTests(TestCaseWithSFTPServer):
     """Tests for upgrade over sftp."""
@@ -175,7 +197,7 @@ finished
         self.assertEqual('', err)
 
 
-class UpgradeRecommendedTests(TestCaseInTempDir):
+class UpgradeRecommendedTests(TestCaseWithTransport):
 
     def test_recommend_upgrade_wt4(self):
         # using a deprecated format gives a warning
@@ -191,3 +213,9 @@ class UpgradeRecommendedTests(TestCaseInTempDir):
         out, err = self.run_bzr('revno a')
         if err.find('upgrade') > -1:
             self.fail("message shouldn't suggest upgrade:\n%s" % err)
+
+    def test_upgrade_shared_repo(self):
+        repo = self.make_repository('repo', format='2a', shared=True)
+        branch = self.make_branch_and_tree('repo/branch', format="pack-0.92")
+        self.get_transport('repo/branch/.bzr/repository').delete_tree('.')
+        out, err = self.run_bzr(['upgrade'], working_dir='repo/branch')
